def markdown_out(added, removed, updated, old_info, new_info, config, changelog_file):
    style = config["format"]["style"]
    available_styles = {
        "bullet": bullet_list,
        "comma": comma_list,
        "ind_bullet": ind_bullet_list,
        "ind_comma": ind_comma_list,
    }
    markdown_text = []
    markdown_text.append(generate_header(old_info, new_info, config))
    markdown_text = available_styles.get(style, bullet_list)(
        added, removed, updated, markdown_text
    )

    if changelog_file is None:
        return str(markdown_text)
    write_to_file(changelog_file, markdown_text)


def write_to_file(filename, text):
    with open(filename, "w", encoding="utf-8") as f:
        f.write(text)


def generate_header(old_info, new_info, config):
    header_format = config["format"]["header"]
    if not header_format["show_header"]:
        return ""

    name = header_format.get("title")
    if name == "auto":
        name = new_info["modpack_name"]

    old_version = (
        f" {old_info['modpack_version']} =>"
        if header_format.get("show_old_version_number")
        else ""
    )
    new_version = (
        f" {new_info['modpack_version']}"
        if header_format.get("show_new_version_number")
        else ""
    )

    return f"{'#' * header_format['size']} {name}{old_version}{new_version}\n"


def bullet_list(added, removed, updated, markdown_text):
    if added:
        markdown_text.extend(["### Added:"] + [f"- {mod}" for mod in added])
    if removed:
        markdown_text.extend(["### Removed:"] + [f"- {mod}" for mod in removed])
    if updated:
        markdown_text.extend(["### Updated:"] + [f"- {mod}" for mod in updated])

    return "\n".join(markdown_text)


def comma_list(added, removed, updated, markdown_text):
    if added:
        markdown_text.append("- Added: " + ", ".join(f"{mod}" for mod in added))
    if removed:
        markdown_text.append("- Removed: " + ", ".join(f"{mod}" for mod in removed))
    if updated:
        markdown_text.append("- Updated: " + ", ".join(f"{mod}" for mod in updated))

    return "\n".join(markdown_text)


def ind_bullet_list(added, removed, updated, markdown_text):
    if added:
        markdown_text.extend(f"- Added {mod}" for mod in added)
    if removed:
        markdown_text.extend(f"- Removed {mod}" for mod in removed)
    if updated:
        markdown_text.extend(f"- Updated {mod}" for mod in updated)

    return "\n".join(markdown_text)


def ind_comma_list(added, removed, updated, markdown_text):
    markdown_text.append("**Changes:**")

    if added:
        markdown_text.append(", ".join(f"Added {mod}" for mod in added))
    if removed:
        markdown_text.append(", ".join(f"Removed {mod}" for mod in removed))
    if updated:
        markdown_text.append(", ".join(f"Updated {mod}" for mod in updated))

    return " ".join(markdown_text)



import asyncio
import json

import aiohttp

from .utils import CF_API_URL, CF_HEADERS, MR_API_URL, MR_HEADERS


async def get_mod_names(MODPACKS_FORMAT, added_ids, removed_ids, updated_ids):
    async with aiohttp.ClientSession(
        timeout=aiohttp.ClientTimeout(total=15)
    ) as session:
        api_function = {
            "modrinth": request_from_mr_api,
            "curseforge": request_from_cf_api,
        }.get(MODPACKS_FORMAT)
        added_names, removed_names, updated_names = await asyncio.gather(
            api_function(session, added_ids),
            api_function(session, removed_ids),
            api_function(session, updated_ids),
        )

    return added_names, removed_names, updated_names


async def request_from_mr_api(session, ids):
    if not ids:
        return []

    names = []
    url = f"{MR_API_URL}/projects?ids={json.dumps(list(ids))}"

    async with session.get(url, headers=MR_HEADERS) as response:
        response.raise_for_status()
        data = await response.json()
        names = [project.get("title") for project in data]
    return names


async def request_from_cf_api(session, ids):
    if not ids:
        return []

    names = []
    url = f"{CF_API_URL}v1/mods"

    async with session.post(
        url, headers=CF_HEADERS, json={"modIds": list(ids)}
    ) as response:
        data = await response.json()
        names = [project["name"] for project in data["data"]]
    return names



# This file is a CLI wrapper for the modpack_changelogger function in main.py
# It provides a CLI interface for the package

import aiohttp
import click

from modpack_changelogger.main import generate_changelog
from modpack_changelogger.utils import (
    ConfigValidationError,
    DifferentModpackFormatError,
    ModpackFormatError,
    UnsupportedModpackFormatError,
    create_config,
)
from modpack_changelogger.version import __version__

CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help", "-?"])


def handle_error(error, message=None):
    """Centralized error handler for CLI errors."""
    if message:
        click.secho(f"Error: {message}", err=True, fg="bright_red")
        click.secho(f"Details: {error}", err=True, fg="red")
    else:
        click.secho(f"Error: {error}", err=True, fg="bright_red")


@click.group(invoke_without_command=True, context_settings=CONTEXT_SETTINGS)
@click.option("-v", "--version", is_flag=True, help="Show the version and exit")
@click.option("-o", "--old", help="First pack to compare")
@click.option("-n", "--new", help="The pack to compare against")
@click.option("-c", "--config", help="Use a config file")
@click.option("-f", "--file", help="Specify the output file for the changelog")
@click.pass_context
def cli(ctx, version, old, new, config, file):
    """CLI wrapper for Modpack Changelogger."""
    # Show help and exit if no arguments are provided
    if ctx.invoked_subcommand is None and not any([version, old, new, config, file]):
        click.echo(ctx.get_help())
        ctx.exit()

    if ctx.invoked_subcommand is not None:
        return

    if version:
        click.echo(f"Modpack Changelogger {__version__}")
        return

    if not (old and new):
        handle_error(
            RuntimeError(
                "Both old and new modpacks must be provided for the comparison."
            )
        )
        return

    try:
        generate_changelog(old, new, config, file)
        click.secho(f"Changelog successfully generated in '{file}'.", fg="green")
    except FileNotFoundError as error:
        handle_error(error)
    except PermissionError as error:
        handle_error(error)
    except (
        TimeoutError,
        aiohttp.ClientConnectionError,
        aiohttp.ClientResponseError,
    ) as error:
        handle_error(
            error,
            "Unable to connect to the API. This could be due to internet connection issues, API downtime, or incorrect URLs. Please verify and try again.",
        )
    except (
        UnsupportedModpackFormatError,
        DifferentModpackFormatError,
        ModpackFormatError,
        ConfigValidationError,
    ) as error:
        handle_error(error, str(error))
    except Exception as error:
        handle_error(error, f"Unhandled error: {error!r}")


@cli.command(context_settings=CONTEXT_SETTINGS)
def newconfig():
    """Create a new configuration file."""
    try:
        create_config()
        click.secho(
            "A new configuration file has been created successfully.", fg="green"
        )
    except Exception as error:
        handle_error(error, "Unable to create a new configuration file.")


if __name__ == "__main__":
    cli()



import asyncio

from .get_mod_names import get_mod_names


def compare_packs(
    MODPACKS_FORMAT,
    old_ids,
    new_ids,
    old_info,
    new_info,
    old_config_hash,
    new_config_hash,
    old_script_hash,
    new_script_hash,
    config,
):
    updated_ids = old_ids & new_ids
    added_ids = new_ids - old_ids
    removed_ids = old_ids - new_ids

    # Remove a category if disabled in config
    added_ids = added_ids if config["check"]["added_mods"] else set()
    removed_ids = removed_ids if config["check"]["removed_mods"] else set()
    updated_ids = updated_ids if config["check"]["updated_mods"] else set()

    added_mods, removed_mods, updated_mods = asyncio.run(
        get_mod_names(MODPACKS_FORMAT, added_ids, removed_ids, updated_ids)
    )

    added_mods = sorted(mod for mod in added_mods if mod)
    removed_mods = sorted(mod for mod in removed_mods if mod)
    updated_mods = sorted(mod for mod in updated_mods if mod)

    if config["check"]["loader"]:
        if old_info["loader"] != new_info["loader"]:
            added_mods.append(f"{new_info['loader'].capitalize()} (mod loader)")
            removed_mods.append(f"{old_info['loader'].capitalize()} (mod loader)")

        elif old_info["loader_version"] != new_info["loader_version"]:
            updated_mods.append(f"{new_info['loader'].capitalize()} (mod loader)")

    if (
        config["check"]["mc_version"]
        and old_info["mc_version"] != new_info["mc_version"]
    ):
        updated_mods.append(f"Minecraft version to {new_info['mc_version']}")

    if config["check"]["config"] and old_config_hash != new_config_hash:
        updated_mods.append("Mods configs")

    if config["check"]["script"] and old_script_hash != new_script_hash:
        updated_mods.append("CraftTweaker scripts")

    return added_mods, removed_mods, updated_mods



import os
import sys

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "../..")))
from modpack_changelogger import generate_changelog

old_pack = r"test/packs/old.mrpack"
new_pack = r"test/packs/new.mrpack"
name = "comma"

json_path = os.path.join(r"test/configs", f"{name}.json")
if not os.path.isfile(json_path):
    json_path = None
md_path = os.path.join(r"test/expected", f"{name}.md")

generate_changelog(old_pack, new_pack, json_path, md_path)



import re


def mr_get_pack_data(old_json, new_json):
    pattern = re.compile(r"(?<=data\/)[a-zA-Z0-9]{8}")

    def get_dependency_info(json):
        loader = next(
            (key for key in json.get("dependencies", {}) if key != "minecraft"),
            "Unknown",
        )
        return {
            "modpack_name": json.get("name", "Unknown"),
            "modpack_version": json.get("versionId", "Unknown"),
            "mc_version": json.get("dependencies", {}).get("minecraft", "Unknown"),
            "loader": loader,
            "loader_version": json.get("dependencies", {}).get(loader, "Unknown"),
        }

    def get_mod_urls(json):
        return [download for url in json["files"] for download in url["downloads"]]

    def extract_mod_ids(url_list):
        return [pattern.search(str(url)).group(0) for url in url_list]

    old_info, new_info = get_dependency_info(old_json), get_dependency_info(new_json)
    new_urls, old_urls = set(get_mod_urls(new_json)), set(get_mod_urls(old_json))
    # remove urls that are in both packs (not added nor removed nor updated)
    common_urls = new_urls & old_urls
    new_urls -= common_urls
    old_urls -= common_urls

    old_ids, new_ids = set(extract_mod_ids(old_urls)), set(extract_mod_ids(new_urls))
    return old_ids, new_ids, old_info, new_info


def cf_get_pack_data(old_json, new_json):
    def get_dependency_info(json):
        loader_string = (
            json.get("minecraft", {})
            .get("modLoaders", [{}])[0]
            .get("id", "Unknown-Unknown")
        )
        loader, loader_version = (
            loader_string.split("-", 1)
            if "-" in loader_string
            else ("Unknown", "Unknown")
        )
        return {
            "modpack_name": json.get("name", "Unknown"),
            "modpack_version": json.get("version", "Unknown"),
            "mc_version": json.get("minecraft", {}).get("version", "Unknown"),
            "loader": loader,
            "loader_version": loader_version,
        }

    def get_mod_ids(json):
        # Extracts the file and project IDs from the JSON
        return {item["fileID"]: item["projectID"] for item in json["files"]}

    old_file_ids = get_mod_ids(old_json)
    new_file_ids = get_mod_ids(new_json)

    # To reference a file cf has a project id (the mod) and a file id (the version)
    # with this code we can get mod that have been changed between the two packs, so with unquie file ids
    old_ids = {
        old_file_ids[file_id] for file_id in old_file_ids if file_id not in new_file_ids
    }
    new_ids = {
        new_file_ids[file_id] for file_id in new_file_ids if file_id not in old_file_ids
    }

    old_info, new_info = get_dependency_info(old_json), get_dependency_info(new_json)
    return old_ids, new_ids, old_info, new_info



import hashlib
import json
import os
import tempfile
import zlib
from zipfile import BadZipFile, ZipFile

from .utils import (
    DifferentModpackFormatError,
    ModpackFormatError,
    UnsupportedModpackFormatError,
)


def get_json(MODPACKS_FORMAT, path):
    if not os.path.exists(path):
        raise FileNotFoundError(f"The file '{path}' does not exist.")

    if path.endswith(".mrpack"):
        if MODPACKS_FORMAT == "curseforge":
            raise DifferentModpackFormatError("curseforge", "modrinth")
        MODPACKS_FORMAT = "modrinth"

    elif path.endswith(".zip"):
        if MODPACKS_FORMAT == "modrinth":
            raise DifferentModpackFormatError("modrinth", "curseforge")
        MODPACKS_FORMAT = "curseforge"

    else:
        raise UnsupportedModpackFormatError(path)

    # Create a temporary directory to extract the modpack into
    with tempfile.TemporaryDirectory() as temp_dir:
        try:
            # Unpack the modpack file into the temp directory
            with ZipFile(path, "r") as zip_obj:
                zip_obj.extractall(path=temp_dir)

            # Get config folder hash
            config_hash = hash_directory(os.path.join(temp_dir, "overrides", "config"))
            script_hash = hash_directory(os.path.join(temp_dir, "overrides", "scripts"))
            overrides_name = get_overrides(os.path.join(temp_dir, "overrides", "mods"))

            # Parse the json file
            json_path = os.path.join(
                temp_dir,
                "modrinth.index.json"
                if MODPACKS_FORMAT == "modrinth"
                else "manifest.json",
            )
            with open(json_path, encoding="utf-8") as json_file:
                return (
                    MODPACKS_FORMAT,
                    json.load(json_file),
                    config_hash,
                    script_hash,
                    overrides_name,
                )
        except FileNotFoundError as error:
            raise ModpackFormatError(
                path, "missing manifest.json or modrinth.index.json"
            ) from error
        except json.JSONDecodeError as error:
            raise ModpackFormatError(
                path, "invalid manifest.json or modrinth.index.json"
            ) from error
        except BadZipFile as error:
            raise ModpackFormatError(
                path, "corrupted or invalid compression of the mrpack file"
            ) from error


def hash_directory(directory):
    crc = 0
    chunk_size = 4096  # Read files in 4KB chunks
    for dirpath, _, filenames in os.walk(directory):
        for filename in sorted(filenames):  # Sort filenames for consistent hashing
            filepath = os.path.join(dirpath, filename)
            try:
                with open(filepath, "rb") as f:
                    while True:
                        chunk = f.read(chunk_size)
                        if not chunk:
                            break
                        crc = zlib.crc32(chunk, crc)
            except OSError:
                continue
    return f"{crc & 0xFFFFFFFF:08x}"  # Return CRC32 as an 8-character hex string


def calculate_hash(filename):
    hash_sha512 = hashlib.sha512()
    chunk_size = 4096  # Read files in 4KB chunks
    with open(filename, "rb") as f:
        while True:
            chunk = f.read(chunk_size)
            if not chunk:
                break
            hash_sha512.update(chunk)
    return hash_sha512.hexdigest()


def get_overrides(directory):
    hash_dict = {}
    if not os.path.exists(directory):
        return hash_dict  # Return empty dict if directory doesn't exist
    for root, _, files in os.walk(directory):
        for file in files:
            filepath = os.path.join(root, file)
            file_hash = calculate_hash(filepath)
            hash_dict[file] = file_hash
    return hash_dict



import json
import os

from .utils import DEFAULT_CONFIG, ConfigValidationError


def validate_config(config, default_config):
    """Check if all the fields are present and are of the correct type"""

    for key, default_value in default_config.items():
        if key not in config:
            config[key] = default_value
        elif isinstance(default_value, dict):  # Recursively validate nested keys
            config[key] = validate_config(config.get(key, {}), default_value)
        elif not isinstance(config[key], type(default_value)):
            raise ConfigValidationError(
                f'Config field "{key}" must be of type {type(default_value).__name__}'
            )
    return config


def load_config(config_file):
    if not config_file:
        return DEFAULT_CONFIG
    if not os.path.exists(config_file):
        raise FileNotFoundError(f"The file '{config_file}' does not exist.")

    with open(config_file, encoding="utf-8") as f:
        config = json.load(f)
        config = validate_config(config, DEFAULT_CONFIG)

    return config



import asyncio

import aiohttp

from .utils import MR_API_URL, MR_HEADERS


def add_overrides(MODPACKS_FORMAT, old_overrides, new_overrides, config):
    identical_entries = set(old_overrides.values()) & set(new_overrides.values())
    old_overrides = {
        file_hash: name
        for file_hash, name in old_overrides.items()
        if name not in identical_entries
    }
    new_overrides = {
        file_hash: name
        for file_hash, name in new_overrides.items()
        if name not in identical_entries
    }

    async def get_names_from_hashes(dict1, dict2):
        async with aiohttp.ClientSession(
            timeout=aiohttp.ClientTimeout(total=15)
        ) as session:
            for d in [dict1, dict2]:
                for file_hash in list(d.keys()):
                    url = f"{MR_API_URL}/version_file/{d[file_hash]}"

                    async with session.get(url, headers=MR_HEADERS) as response:
                        if response.status == 200:
                            data = await response.json()
                            project_name = data["project_id"]
                            d[project_name] = d.pop(file_hash)
                            d[project_name] = True
                        else:
                            d[file_hash] = False

    if MODPACKS_FORMAT == "modrinth":
        asyncio.run(get_names_from_hashes(old_overrides, new_overrides))

    old_identified_overrides = {
        name for name, key in old_overrides.items() if key is True
    }
    old_unidentified_overrides = {
        name for name, key in old_overrides.items() if key is False
    }
    new_identified_overrides = {
        name for name, key in new_overrides.items() if key is True
    }
    new_unidentified_overrides = {
        name for name, key in new_overrides.items() if key is False
    }

    if (
        config["check"]["identified_overrides_mods"]
        and not config["check"]["unidentified_overrides_mods"]
    ):
        return old_identified_overrides, new_identified_overrides, {}, {}
    if (
        not config["check"]["identified_overrides_mods"]
        and config["check"]["unidentified_overrides_mods"]
    ):
        return {}, {}, old_unidentified_overrides, new_unidentified_overrides

    return (
        old_identified_overrides,
        new_identified_overrides,
        old_unidentified_overrides,
        new_unidentified_overrides,
    )



# This section of the file contains some hardcoded values, do not edit it directly if you don't know what you are doing.

# Import version number from version.py
from .version import __version__ as VERSION

DEFAULT_CONFIG = {
    "check": {
        "added_mods": True,
        "removed_mods": True,
        "updated_mods": True,
        "loader": True,
        "mc_version": False,
        "config": False,
        "script": False,
        "identified_overrides_mods": True,
        "unidentified_overrides_mods": False,
    },
    "format": {
        "style": "bullet",
        "header": {
            "show_header": True,
            "size": 2,
            "title": "auto",
            "show_old_version_number": False,
            "show_new_version_number": True,
        },
    },
}

MR_HEADERS = {
    "User-Agent": f"TheBossMagnus/modpack-changelogger/{VERSION} (thebossmagnus@proton.me)"
}
MR_API_URL = "https://api.modrinth.com/v2"

# DO NOT USE THIS KEY FOR YOUR OWN PROJECT/FORKS
CF_KEY = "$2a$10$GiT8VjJE8VJpcK68Wlz6aeJ5CPAZcRuTBcGuys8XtX5hGC87sIgku"
# You can get your own key at https://docs.curseforge.com

CF_HEADERS = {
    "x-api-key": CF_KEY,
    "Content-Type": "application/json",
    "Accept": "application/json",
}
CF_API_URL = "https://api.curseforge.com/"

# This section of the file contains some utility functions and exception.


def create_config():
    import json

    with open("config.json", "w", encoding="utf-8") as f:
        json.dump(DEFAULT_CONFIG, f, indent=4)


class UnsupportedModpackFormatError(Exception):
    """Exception raised for unsupported modpack formats."""

    def __init__(self, path):
        self.path = path
        super().__init__(
            f"The modpack '{path}' is not in a supported format (mrpack or zip). "
        )


class DifferentModpackFormatError(Exception):
    """Exception raised for different modpack formats."""

    def __init__(self, old_format, new_format):
        self.old_format = old_format
        self.new_format = new_format
        super().__init__(
            f"Both modpacks must be in the same format (old: {old_format}, new: {new_format})."
        )


class ModpackFormatError(Exception):
    """Exception raised when the modpack is wrongly formatted."""

    def __init__(self, path, error):
        self.path = path
        self.error = error
        super().__init__(f"The modpack '{path}' is not packed correctly ({error}).")


class ConfigValidationError(ValueError):
    """Exception raised for invalid config values."""

    def __init__(self, message):
        super().__init__(message)



"""
Main entry point for running Modpack Changelogger as a module.

This allows the package to be imported in other Python scripts.
"""

from modpack_changelogger.cli_wrapper import main

if __name__ == "__main__":
    main()



"""
Modpack Changelogger - A powerful and customizable Python tool to generate a changelog between two Minecraft modpacks.
"""

from .main import generate_changelog
from .version import __version__

__all__ = ["__version__", "generate_changelog"]



import filecmp
import io
import os
import re
import subprocess
import sys

import pytest

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), "..")))

from modpack_changelogger import generate_changelog  # pylint:disable=C0413

OLD_PACK = "test/packs/old.mrpack"
NEW_PACK = "test/packs/new.mrpack"


def test_mr():
    expected_output = "test/expected/mr.md"

    generate_changelog(OLD_PACK, NEW_PACK, None, None)
    assert filecmp.cmp("Changelog.md", expected_output, shallow=False)


def test_cf():
    expected_output = "test/expected/cf.md"

    generate_changelog("test/packs/old.zip", "test/packs/new.zip", None, None)
    assert filecmp.cmp("Changelog.md", expected_output, shallow=False)

def test_cli():
    expected_output = "test/expected/cli.md"

    result = subprocess.run(
        ["modpack-changelogger", "-o", OLD_PACK, "-n", NEW_PACK, "-f", "Changelog.md"],
        check=False,
    )
    assert result.returncode == 0
    assert filecmp.cmp("Changelog.md", expected_output, shallow=False)


def test_custom_changelog_file():
    expected_output = "test/expected/custom_changelog_file.md"

    try:
        generate_changelog(OLD_PACK, NEW_PACK, None, "custom_name.md")
        assert filecmp.cmp("custom_name.md", expected_output, shallow=False)
    finally:
        if os.path.exists("custom_name.md"):
            os.remove("custom_name.md")


def test_formatted():
    expected_output = "test/expected/formatted.md"
    with open(expected_output, "r", encoding="utf-8") as file:
        expected_output = file.read()

    assert expected_output == generate_changelog(OLD_PACK, NEW_PACK, None, "formatted")


def test_unformatted():
    expected_output_file = "test/expected/unformatted.md"

    output1, output2, output3 = generate_changelog(OLD_PACK, NEW_PACK, None, "unformatted")

    with open(expected_output_file, "r", encoding="utf-8") as f:
        expected1, expected2, expected3 = f.read().split("\n")

    assert str(output1) == expected1.strip()
    assert str(output2) == expected2.strip()
    assert str(output3) == expected3.strip()


def test_console():
    expected_output_file = "test/expected/console.md"

    console_output = io.StringIO()
    sys.stdout = console_output
    generate_changelog(OLD_PACK, NEW_PACK, None, "console")
    sys.stdout = sys.__stdout__
    console_output = console_output.getvalue()

    with open(expected_output_file, "r", encoding="utf-8") as file:
        expected_output = file.read()

    assert expected_output == console_output


def test_config_new():
    try:
        result = subprocess.run(["modpack-changelogger", "newconfig"], check=False, stdout=subprocess.PIPE)
        assert result.returncode == 0
        assert os.path.isfile("config.json")
        assert result.stdout.decode("utf-8") == f"A new configuration file has been created successfully.{os.linesep}"
    finally:
        os.remove("config.json")


def test_version():

    result = subprocess.run(["modpack-changelogger", "-v"], check=False, capture_output=True, text=True)
    assert re.match(r"Modpack Changelogger \d+\.\d+\.\d+(-\w+)?", result.stdout.strip()) is not None

    assert result.returncode == 0


def test_check_options():
    config_path = "test/configs/check_options.json"
    expected_output = "test/expected/check_options.md"

    generate_changelog(OLD_PACK, NEW_PACK, config_path, None)
    assert filecmp.cmp("Changelog.md", expected_output, shallow=False)


def test_comma():
    config_path = "test/configs/comma.json"
    expected_output = "test/expected/comma.md"

    generate_changelog(OLD_PACK, NEW_PACK, config_path, None)
    assert filecmp.cmp("Changelog.md", expected_output, shallow=False)


def test_ind_bullet():
    config_path = "test/configs/ind_bullet.json"
    expected_output = "test/expected/ind_bullet.md"

    generate_changelog(OLD_PACK, NEW_PACK, config_path, None)
    assert filecmp.cmp("Changelog.md", expected_output, shallow=False)


def test_ind_comma():
    config_path = "test/configs/ind_comma.json"
    expected_output = "test/expected/ind_comma.md"

    generate_changelog(OLD_PACK, NEW_PACK, config_path, None)
    assert filecmp.cmp("Changelog.md", expected_output, shallow=False)


def test_header():
    config_path = "test/configs/header.json"
    expected_output = "test/expected/header.md"

    generate_changelog(OLD_PACK, NEW_PACK, config_path, None)
    assert filecmp.cmp("Changelog.md", expected_output, shallow=False)


def test_broken_config():
    config_path = "test/configs/broken_config.json"

    with pytest.raises(ValueError):
        generate_changelog(OLD_PACK, NEW_PACK, config_path, None)


def test_inavlid_config_path():
    config_path = "invalid_config.json"

    with pytest.raises(FileNotFoundError):
        generate_changelog(OLD_PACK, NEW_PACK, config_path, None)



from .compare_packs import compare_packs
from .config_handler import load_config
from .extract_pack_data import cf_get_pack_data, mr_get_pack_data
from .get_json import get_json
from .out import markdown_out
from .overrides_detection import add_overrides


def generate_changelog(old_path, new_path, config_path, changelog_file):
    # Load config
    config = load_config(config_path)

    # Parse the json files
    MODPACKS_FORMAT, old_json, old_config_hash, old_script_hash, old_overrides = (
        get_json(None, old_path)
    )  # None because that the format is not yet known, it will be whatever detected by the function
    MODPACKS_FORMAT, new_json, new_config_hash, new_script_hash, new_overrides = (
        get_json(MODPACKS_FORMAT, new_path)
    )  # The format of the first pack is now known, so the function checks it is the same as the second one

    # Get pack data based on the modpack format
    if MODPACKS_FORMAT == "modrinth":
        old_ids, new_ids, old_info, new_info = mr_get_pack_data(old_json, new_json)
    else:
        old_ids, new_ids, old_info, new_info = cf_get_pack_data(old_json, new_json)
    if (
        config["check"]["identified_overrides_mods"]
        or config["check"]["unidentified_overrides_mods"]
    ):
        (
            old_identified_overrides,
            new_identified_overrides,
            old_unidentified_overrides,
            new_unidentified_overrides,
        ) = add_overrides(MODPACKS_FORMAT, old_overrides, new_overrides, config)
        old_ids = old_ids.union(old_identified_overrides)
        new_ids = new_ids.union(new_identified_overrides)

    # Compare the packs
    added, removed, updated = compare_packs(
        MODPACKS_FORMAT,
        old_ids,
        new_ids,
        old_info,
        new_info,
        old_config_hash,
        new_config_hash,
        old_script_hash,
        new_script_hash,
        config,
    )
    if config["check"]["unidentified_overrides_mods"]:
        added.extend(new_unidentified_overrides)
        removed.extend(old_unidentified_overrides)

    if changelog_file is None:
        changelog_file = "Changelog.md"

    if changelog_file.lower() == "unformatted":
        return added, removed, updated

    if changelog_file.lower() == "formatted":
        return markdown_out(
            added, removed, updated, old_info, new_info, config, None
        )  # if changelog_file is None, it will return the markdown text

    if changelog_file.lower() == "console":
        print(markdown_out(added, removed, updated, old_info, new_info, config, None))
    else:
        markdown_out(
            added, removed, updated, old_info, new_info, config, changelog_file
        )
